<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:in="http://www.wwp.brown.edu/ns/yaps/1.0" xmlns:y="http://www.wwp.brown.edu/ns/yaps/1.0"
  xmlns="http://www.w3.org/1999/xhtml" version="1.0">

  <xsl:output method="html" encoding="UTF-8" indent="yes"
    doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
  <xsl:strip-space elements="*"/>
  <xsl:preserve-space elements="eg egXML egMarckup pre y:eg y:egXML y:egMarkup"/>

  <xsl:param name="myTitle">
    <xsl:choose>
      <xsl:when test="/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:title[not(@type)]">
        <xsl:value-of
          select="/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:title[not(@type)][1]//text()"/>
      </xsl:when>
      <xsl:when test="/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:title[@type='main']">
        <xsl:value-of
          select="/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:title[@type='main'][1]//text()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message>WARNING: no appropriate title found in TEI/teiHeader/fileDesc/titleStmt/, and
          none supplied as 'myTitle' parameter</xsl:message>
        <xsl:value-of select="'YAPS file'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>
  <xsl:param name="copyrightText">
    <!-- If the user hasn't provided a name, we try to extract it from the <teiHeader> using -->
    <!-- mildly complicated heuristics. -->
    <xsl:variable name="publText" select="normalize-space(/in:TEI/in:teiHeader/in:fileDesc/in:publicationStmt)"/>
    <xsl:choose>
      <xsl:when test="contains( $publText, 'Creative Commons' )">
        <xsl:text>Available under Creative Commons; see source for details</xsl:text>
      </xsl:when>
      <xsl:when test="contains( $publText,'&#xA9; ')">
        <xsl:value-of select="$publText"/>
      </xsl:when>
      <xsl:when test="contains( $publText,'(c) ')">
        <xsl:value-of select="concat('©', substring-after( $publText, '(c) ') )"/>
      </xsl:when>
      <xsl:when test="contains( $publText,'copyleft ')">
        <xsl:value-of select="concat('©', substring-after( $publText, 'copyleft ') )"/>
      </xsl:when>
      <xsl:when test="contains( $publText,'copyright ')">
        <xsl:value-of select="concat('©', substring-after( $publText, 'copyright ') )"/>
      </xsl:when>
      <xsl:when test="contains( $publText,'Copyleft ')">
        <xsl:value-of select="concat('©', substring-after( $publText, 'Copyleft ') )"/>
      </xsl:when>
      <xsl:when test="contains( $publText,'Copyright ')">
        <xsl:value-of select="concat('©', substring-after( $publText, 'Copyright ') )"/>
      </xsl:when>
      <xsl:otherwise>        
        <xsl:text>unable to derive copyleft holder from source, you’d better ask</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>
  <xsl:param name="byline">
    <!-- The following code handles getting the punctuation (commas and "and") -->
    <!-- in the list of authors, and provides a default if there is no <author>. -->
    <!-- Note that in the case of more than 2 authors this algorithm inserts an -->
    <!-- extra blank between the ultimate comma and the "and", but since every -->
    <!-- single browser I know of normalizes such spaces, I'm not worried about -->
    <!-- it. -->
    <xsl:variable name="numAuthors"
      select="count(/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:author)"/>
    <xsl:if test="$numAuthors=0">Brown WWP</xsl:if>
    <xsl:for-each select="/in:TEI/in:teiHeader/in:fileDesc/in:titleStmt/in:author">
      <xsl:if test="$numAuthors>1 and position()=last()">
        <xsl:text> and </xsl:text>
      </xsl:if>
      <xsl:apply-templates/>
      <xsl:if test="$numAuthors>2 and position()&lt;last()">
        <xsl:text>, </xsl:text>
        <br/>
      </xsl:if>
    </xsl:for-each>
  </xsl:param>

  <!-- **************************************** -->
  <!-- match root, generate <html> output shell -->
  <!-- **************************************** -->
  <xsl:template match="/">
    <xsl:variable name="lang">
      <xsl:choose>
        <xsl:when test="y:TEI/@xml:lang">
          <xsl:value-of select="y:TEI/@xml:lang"/>
        </xsl:when>
        <xsl:otherwise>en</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <html lang="{$lang}" xml:lang="{$lang}">
      <xsl:text>&#x0A;</xsl:text>
      <xsl:comment>This is a derived file, DO NOT EDIT</xsl:comment>
      <xsl:apply-templates select="y:TEI/y:teiHeader"/>
      <xsl:apply-templates select="y:TEI/y:text"/>
    </html>
  </xsl:template>

  <xsl:template match="y:teiHeader">
    <head>
      <title>
        <xsl:value-of select="$myTitle"/>
      </title>
      <link rel="stylesheet" type="text/css" media="screen, projection" href="./slidy/styles/slidy.css"/>
      <link rel="stylesheet" href="./slidy/styles/wwp-slidy.css" type="text/css"/>
      <script charset="utf-8" type="text/javascript" src="./slidy/scripts/slidy.js"/>
      <meta name="copyright" content="{$copyrightText}"/>
      <!-- to activate timer insert a 
        <meta name="duration" content="M"/>
            element, where M = a number of minutes for countdown timer -->
    </head>
  </xsl:template>
  
  <!-- we only match <publicationStmt> when it's pulled in to 1st slide -->
  <xsl:template match="y:publicationStmt">
    <xsl:choose>
      <xsl:when test="y:availability">
        <xsl:apply-templates select="y:availability/*"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="y:text">
    <xsl:apply-templates select="y:presentation"/>
  </xsl:template>

  <xsl:template match="y:presentation">
    <body>
      <div class="background">
        <!-- I have no idea why the following image looks a tiny bit better with -->
        <!-- width=65, as it is only 61 px wide. -->
        <img id="head-icon" src="../common/WWPlogo.png" alt="WWP logo" width="65px"
          height="53px"/>
        <xsl:comment>
          <!-- Julia doesn't like the upper-right corner logo -->
          <img id="head-logo" src="./WWPbanner.png" alt="WWP banner"
            height="53px" width="260"/>
        </xsl:comment>
      </div>
      <div class="slide cover">
        <img src="../common/WWPlogo.png" alt="WWP logo" height="53px" class="cover"/>
        <br clear="all"/>
        <h1 class="slideTitle">
          <xsl:value-of select="$myTitle"/>
        </h1>
        <p class="byline">
          <xsl:value-of select="$byline"/>
        </p>
        <div class="availability">
          <xsl:apply-templates select="/in:TEI/in:teiHeader/in:fileDesc/in:publicationStmt"/>
        </div>
      </div>
      <xsl:apply-templates select="y:section|y:sectionGrp/y:section"/>
    </body>
  </xsl:template>

  <xsl:template match="y:section">
    <xsl:apply-templates select="./y:slide"/>
    <xsl:apply-templates select="./y:lectureNote"/>
    <xsl:apply-templates select="./y:handout"/>
  </xsl:template>
  
  <xsl:template match="y:slide">
    <div class="slide">
      <xsl:variable name="heading">
        <xsl:choose>
          <xsl:when test="ancestor::y:sectionGrp">
            <xsl:apply-templates select="../../y:head"/>
            <xsl:text>: </xsl:text>
            <xsl:apply-templates select="../y:head"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="../y:head"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <h1>
        <xsl:value-of select="$heading"/>
      </h1>
      <div>
        <xsl:if test="contains( normalize-space(self::*/@*|child::*/@*), 'incremental')">
          <xsl:attribute name="class">incremental</xsl:attribute>
        </xsl:if>
        <xsl:apply-templates select="*"/>
      </div>
    </div>
  </xsl:template>
  
  <xsl:template match="y:handout | y:lectureNote">
    <div class="handout">
      <xsl:variable name="heading">
        <xsl:choose>
          <xsl:when test="ancestor::y:sectionGrp">
            <xsl:apply-templates select="../../y:head"/>
            <xsl:text>: </xsl:text>
            <xsl:apply-templates select="../y:head"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="../y:head"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <h1>
        <xsl:value-of select="$heading"/>
      </h1>
      <div>
        <xsl:apply-templates select="*"/>
      </div>
    </div>
  </xsl:template>
  
  <xsl:template match="y:p">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">p</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="in:argument | in:epigraph">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">div</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- the following should contain all the members of model.inter in the input schema that -->
  <!-- do not have templates of their own elsewhere -->
  <!-- Note: I think the <text> not a child of <TEI> will be replaced by <floatingText> soon -->
  <xsl:template
    match="y:cit|y:label|y:bibl|y:q|y:quote|y:said|y:text[not(parent::y:TEI)]">
    <!-- <figure>, <list>, and <table> are matched elsewhere -->
    <xsl:choose>
      <!-- if it's in something for which we've already generated a <p>, put -->
      <!-- it in a <span>: -->
      <xsl:when
        test="ancestor::in:p or ancestor::in:item or ancestor::in:head or ancestor::in:table or ancestor::in:cit">
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">span</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <!-- otherwise, put it in a <p>: -->
      <xsl:otherwise>
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">p</xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- All phrase-level elements get <span>: -->
  <xsl:template
    match="
       in:address
     | in:att
     | in:author
     | in:biblScope
     | in:code
     | in:date
     | in:distributor
     | in:edition
     | in:editor
     | in:emph
     | in:foreign
     | in:formula
     | in:gi
     | in:gloss
     | in:hi
     | in:ident
     | in:idno
     | in:mentioned
     | in:name
     | in:num
     | in:publisher
     | in:pubPlace
     | in:rs
     | in:soCalled
     | in:tag
     | in:term
     | in:title
     | in:val
     ">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">span</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- kinda icky, but use <lb> for formatting in some -->
  <!-- circumstances: -->
  <xsl:template match="in:lb">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">br</xsl:with-param>
      <xsl:with-param name="useClass">no</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- *********************** -->
  <!-- subroutines, as it were -->
  <!-- *********************** -->
  <xsl:template name="transformMe">
    <!-- 
      I think I may change how this works someday soon, but for now:
      if rend="class(duck)" was specified in the source, then
      we set class=duck on output
      else if param(useClass)=no, then
      no class= specified on output
      else if param(useClass)=yes (the default), then
      if no type= specified in source, then set class= value is param(from)
      if type= was specified in source, then
      set class= to param(from) concatonated with "_" and the value of type= 
      else param(useClass)=[something else], then
      set class= to [something else]
    -->
    <xsl:param name="to">p</xsl:param>
    <xsl:param name="useClass">yes</xsl:param>
    <xsl:element name="{$to}">
      <xsl:call-template name="transformAttrs">
        <xsl:with-param name="useClass" select="$useClass"/>
      </xsl:call-template>
      <xsl:apply-templates select="node()[not(self::in:head)]"/>
    </xsl:element>
  </xsl:template>

  <xsl:template name="beforeMe">
    <xsl:param name="put">br</xsl:param>
    <xsl:param name="useClass">yes</xsl:param>
    <xsl:element name="{$put}">
      <xsl:call-template name="transformAttrs">
        <xsl:with-param name="useClass" select="$useClass"/>
      </xsl:call-template>
    </xsl:element>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template name="transformAttrs">
    <xsl:param name="useClass"/>
    <xsl:variable name="didClass"/>
    <!-- Handle routine attrs first, then rend=, and then class=. -->
    <!-- This way a specification of rend="class(duck)" overrides -->
    <!-- the normal or useClass specified class=.                 -->
    <!-- This override idea is NOT WORKING; need to re-write ??   -->
    <xsl:copy-of select="@xml:lang|@xml:base"/>
    <xsl:if test="@xml:id">
      <xsl:attribute name="id">
        <xsl:value-of select="@xml:id"/>
      </xsl:attribute>
    </xsl:if>
    <!-- rend= -->
    <xsl:if test="@rend">
      <xsl:variable name="rend" select="normalize-space( @rend )"/>
      <xsl:choose>
        <xsl:when test="starts-with($rend,'css(')">
          <xsl:attribute name="style">
            <xsl:variable name="sans-pre" select="substring-after($rend,'css(')"/>
            <xsl:value-of select="substring($sans-pre,1,string-length($sans-pre)-1)"/>
          </xsl:attribute>
        </xsl:when>
        <xsl:when test="starts-with($rend,'class(')">
          <xsl:attribute name="class">
            <xsl:variable name="sans-pre" select="substring-after($rend,'class(')"/>
            <xsl:variable name="sans" select="substring($sans-pre,1,string-length($sans-pre)-1)"/>
            <xsl:value-of select="normalize-space($sans)"/>
          </xsl:attribute>
        </xsl:when>
      </xsl:choose>
    </xsl:if>
    <!-- class= -->
    <xsl:variable name="class">
      <xsl:choose>
        <!-- the following test is the same as within rend=; there must be a better way -->
        <!-- to do this. -->
        <xsl:when test="starts-with(normalize-space(@rend),'class(')"/>
        <xsl:when test="$useClass='no'"/>
        <xsl:when test="@type">
          <xsl:value-of select="name(.)"/>
          <xsl:text>_</xsl:text>
          <xsl:value-of select="@type"/>
        </xsl:when>
        <xsl:when test="@subtype">
          <xsl:value-of select="@subtype"/>
        </xsl:when>
        <xsl:when test="@role">
          <xsl:value-of select="@role"/>
        </xsl:when>
        <xsl:when test="not($useClass='yes')">
          <xsl:value-of select="$useClass"/>
        </xsl:when>
        <xsl:otherwise>
          <!-- useClass='yes' -->
          <xsl:value-of select="name(.)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="not($class='')">
      <xsl:attribute name="class">
        <xsl:value-of select="$class"/>
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

  <xsl:template match="in:addrLine">
    <xsl:call-template name="beforeMe">
      <xsl:with-param name="put">br</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="in:figure">
    <xsl:element name="img">
      <xsl:call-template name="transformAttrs"/>
      <xsl:attribute name="alt">
        <xsl:value-of select="./in:figDesc"/>
      </xsl:attribute>
      <xsl:attribute name="src">
        <xsl:value-of select="./y:graphic/@url"/>
      </xsl:attribute>
      <xsl:copy-of select="./in:graphic/@height"/>
      <xsl:copy-of select="./in:graphic/@width"/>
    </xsl:element>
  </xsl:template>

  <!-- the children of <choice> need special treatment ... -->
  <xsl:template match="in:abbr|in:expan">
    <!-- In either case make me a <span>, but we use a different -->
    <!-- class= based on whether we're a child of <choice> or not. -->
    <xsl:choose>
      <xsl:when test="parent::in:choice">
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">span</xsl:with-param>
          <xsl:with-param name="useClass">choice</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">span</xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- lists are complicated -->
  <xsl:template match="in:list">
    <xsl:call-template name="childHead"/>
    <xsl:choose>
      <xsl:when test="@type='unordered' or not(@type)">
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">ul</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='ordered'">
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">ol</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@type='gloss'">
        <xsl:element name="dl">
          <xsl:if test="@subtype">
            <xsl:attribute name="class">
              <xsl:value-of select="@subtype"/>
            </xsl:attribute>
          </xsl:if>
          <xsl:apply-templates select="in:label" mode="gloss"/>
        </xsl:element>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="in:item">
    <xsl:element name="li" xml:space="default">
      <xsl:call-template name="transformAttrs">
        <xsl:with-param name="useClass">no</xsl:with-param>
      </xsl:call-template>
      <xsl:if test="@n">
        <xsl:attribute name="value">
          <xsl:value-of select="@n"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="in:label" mode="gloss">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">dt</xsl:with-param>
      <xsl:with-param name="useClass">no</xsl:with-param>
    </xsl:call-template>
    <xsl:apply-templates select="following-sibling::in:item[1]" mode="gloss"/>
  </xsl:template>

  <xsl:template match="in:item" mode="gloss">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">dd</xsl:with-param>
      <xsl:with-param name="useClass">no</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="in:eg|in:egXML">
    <xsl:call-template name="transformMe">
      <xsl:with-param name="to">pre</xsl:with-param>
      <xsl:with-param name="useClass">yes</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="in:table">
    <xsl:call-template name="childHead"/>
    <xsl:element name="table">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates select="node()[not(self::in:head)]"/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="in:row">
    <xsl:element name="tr">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="in:cell">
    <xsl:element name="td">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <!-- special-case my use of <ref> as <bibl> -->
  <xsl:template match="y:cit/y:ref">
    <p>
      <xsl:call-template name="transformAttrs">
        <xsl:with-param name="useClass">citAuthor</xsl:with-param>
      </xsl:call-template>
      <xsl:apply-templates/>
    </p>
  </xsl:template>
  
  <xsl:template match="in:ref">
    <!-- the following test should really be more rigorous: -->
    <!-- we should actually figure out what document is being -->
    <!-- pointed at using resolve-uri(), and then see if that is -->
    <!-- the current document (how?) -->
    <!-- For now, just check to see if 1st char is a "#"; if it is, -->
    <!-- warn user that generated link will probably not work -->
    <xsl:if test="starts-with(@target,'#')">
      <xsl:message>WARNING: a &lt;ref> is not supposed to point within the input document &#x2014; a
        link will be generated, but it probably won't do what you want.</xsl:message>
    </xsl:if>
    <xsl:element name="a">
      <xsl:attribute name="href">
        <xsl:value-of select="@target"/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template name="childHead">
    <xsl:if test="./in:head">
      <xsl:for-each select="./in:head">
        <xsl:call-template name="transformMe">
          <xsl:with-param name="to">p</xsl:with-param>
          <xsl:with-param name="useClass">
            <xsl:value-of select="name(..)"/>
            <xsl:text>-head</xsl:text>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>

  <xsl:template match="in:table">
    <xsl:call-template name="childHead"/>
    <xsl:element name="table">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates select="node()[not(self::in:head)]"/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="in:row">
    <xsl:element name="tr">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="in:cell">
    <xsl:element name="td">
      <xsl:call-template name="transformAttrs"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

</xsl:stylesheet>
